import pytest
from django.contrib import admin
from django.contrib.auth.models import Permission
from django.utils.module_loading import import_string
from django_perm_filter.utils import filter_perms
from django_perm_filter.utils import sort_perms
from django_perm_filter.utils import unregister_models


def test_sort_perms():
    actions = ["view_model1", "change_model1", "add_model2", "delete_model2"]
    expected_result = ["change_model1", "view_model1", "add_model2", "delete_model2"]

    result = sort_perms(actions)

    assert result == expected_result


@pytest.mark.django_db()
def test_filter_perms(settings):
    """Test the filter_perms function."""
    settings.PERM_FILTER = {
        "HIDE_PERMS": [
            # Django built-in apps
            "admin",
            "contenttypes",
            "sessions",
            "sites",
            # All-auth
            "account",
            "socialaccount",
            # Django built-in auth permissions
            "auth.view_permission",
            "auth.add_permission",
            "auth.change_permission",
            "auth.delete_permission",
        ],
    }

    qs = Permission.objects.all()
    # print(sort_perms([perm.codename for perm in qs]))

    results = filter_perms(qs)
    # print("-" * 40)
    # print(sort_perms([perm.codename for perm in results]))

    # Ensure that no permissions with the following codenames are returned
    codenames = [
        # admin permissions
        "add_logentry",
        "change_logentry",
        "delete_logentry",
        "view_logentry",
        # contenttype permissions
        "add_contenttype",
        "change_contenttype",
        "delete_contenttype",
        "view_contenttype",
        # session permissions
        "add_session",
        "change_session",
        "delete_session",
        "view_session",
        # site permissions
        "add_site",
        "change_site",
        "delete_site",
        "view_site"
        # auth permissions
        "view_permission",
        "add_permission",
        "change_permission",
        "delete_permission",
        # all auth (account) permissions
        "add_emailaddress",
        "change_emailaddress",
        "delete_emailaddress",
        "view_emailaddress",
        "add_emailconfirmation",
        "change_emailconfirmation",
        "delete_emailconfirmation",
        "view_emailconfirmation",
        # all auth (socialaccount) permissions
        "add_socialaccount",
        "change_socialaccount",
        "delete_socialaccount",
        "view_socialaccount",
    ]
    assert all(
        not any(codename in perm.codename for codename in codenames) for perm in results
    )


def test_unregister_models(settings):
    """Test the unregister_models function."""
    settings.PERM_FILTER = {
        "UNREGISTER_MODELS": [
            "django.contrib.sites.models.Site",
            # All-auth
            "allauth.account.models.EmailAddress",
            "allauth.socialaccount.models.SocialAccount",
            "allauth.socialaccount.models.SocialApp",
            "allauth.socialaccount.models.SocialToken",
        ],
    }
    unregister_models()

    for model_string in settings.PERM_FILTER["UNREGISTER_MODELS"]:
        model = import_string(model_string)
        assert not admin.site.is_registered(model)


def test_unregister_models_logs_if_not_found(settings, caplog):
    # Test model not found
    settings.PERM_FILTER = {"UNREGISTER_MODELS": ["fake_app.models.FakeModel"]}

    # with pytest.raises(ModuleNotFoundError):
    unregister_models()

    assert (
        "No module named 'fake_app' Ensure the module is installed and/or added to INSTALLED_APPS if need be."
        in caplog.text
    )



from django.apps import AppConfig

from django_perm_filter.utils import unregister_models


class DjangoPermFilterConfig(AppConfig):
    name = "django_perm_filter"
    verbose_name = "Django Perm Filter"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        unregister_models()



import logging

from django.conf import settings
from django.contrib import admin
from django.db.models import QuerySet
from django.utils.module_loading import import_string

from django_perm_filter.settings import api_settings

logger = logging.getLogger(__name__)


def sort_perms(actions):
    # Split each string by "_" and sort based on the second element, then the first
    return sorted(actions, key=lambda x: (x.split("_")[1], x.split("_")[0]))


def filter_perms(qs: QuerySet) -> QuerySet:
    """
    Filters a queryset by excluding permissions specified in the HIDE_PERMS setting.

    Parameters:
    qs (QuerySet): The initial queryset to filter.

    Returns:
    QuerySet: The filtered queryset.
    """
    for perm in api_settings.HIDE_PERMS:
        if "." in perm:
            app, model = perm.split(".")
            qs = qs.exclude(content_type__app_label=app, codename=model)
        else:
            qs = qs.exclude(content_type__app_label=perm)

    return qs


def unregister_models() -> None:
    """
    Unregisters models specified in the UNREGISTER_MODELS setting.

    This function will attempt to import and unregister each model specified in the setting.
    If a model cannot be found or an error occurs during unregistering, a warning will be logged.

    Raises:
    ModuleNotFoundError: If the model module cannot be imported.
    RuntimeError: If an error occurs during unregistering.
    """
    if hasattr(settings, "PERM_FILTER"):
        for model_str in api_settings.UNREGISTER_MODELS:
            try:
                model = import_string(model_str)
                if admin.site.is_registered(model):
                    admin.site.unregister(model)
            except (ModuleNotFoundError, RuntimeError) as e:
                logger.warning(
                    "%s %s",
                    str(e),
                    "Ensure the module is installed and/or added to INSTALLED_APPS if need be.",
                )



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.utils.module_loading import import_string

from django_perm_filter.mixins import PermissionFilterMixin
from django_perm_filter.settings import api_settings

# import contextlib
# from django.contrib.admin.sites import NotRegistered

User = get_user_model()

user_admin = import_string(api_settings.USER_ADMIN)
group_admin = import_string(api_settings.GROUP_ADMIN)


class UserAdmin(PermissionFilterMixin, user_admin):
    """Custom User Admin with permission filtering."""


class GroupAdmin(PermissionFilterMixin, group_admin):
    """Custom Group Admin with permission filtering."""


# Override default registered Admin for User and Group
# with contextlib.suppress(NotRegistered):
#     admin.site.unregister(User)

# with contextlib.suppress(NotRegistered):
#     admin.site.unregister(Group)

# Register the custom User/Group Admin
if admin.site.is_registered(User):
    admin.site.unregister(User)

admin.site.register(User, UserAdmin)

if admin.site.is_registered(Group):
    admin.site.unregister(Group)

admin.site.register(Group, GroupAdmin)



from http import HTTPStatus

from django.urls import reverse


class TestUserAdmin:
    def test_change(self, admin_client, user):
        """Test our PermissionFilterMixin"""
        url = reverse("admin:auth_user_change", kwargs={"object_id": user.pk})
        response = admin_client.get(url)
        assert response.status_code == HTTPStatus.OK


class TestGroupAdmin:
    def test_change(self, admin_client, group):
        """Test our PermissionFilterMixin"""
        url = reverse("admin:auth_group_change", kwargs={"object_id": group.pk})
        response = admin_client.get(url)
        assert response.status_code == HTTPStatus.OK
